import filecmp
import argparse
from pathlib import PurePath, Path
import os
import re
import typing as t
import itertools
import sys
class COLORS:
    OKGREEN = '\033[92m'
    FAIL = '\033[91m'
    WARNING = '\033[93m'
    ENDC = '\033[0m'

def get_color(is_equal: bool) -> str:
    return COLORS.OKGREEN if is_equal else COLORS.FAIL

def read_file(path: Path) -> t.List[str]:
    if not os.path.exists(path):
        return None
    with open(path, 'r') as f:
        return list(itertools.chain(*[line.split("\\n") for line in f.read().split("\n")]))

def compare_query_results(dbname1: str, dbname2: str, query: int) -> bool:
    path1 = Path(f'data', 'mysql', dbname1, f'{query}.results.sql')
    path2 = Path(f'data', 'mysql', dbname2, f'{query}.results.sql')

    if not os.path.exists(path1):
        print(f'{COLORS.WARNING}Warning: Query {query} - File does not exists: {path1}{COLORS.ENDC}')
        return False
    if not os.path.exists(path2):
        print(f'{COLORS.WARNING}Warning: Query {query} - File does not exists: {path2}{COLORS.ENDC}')
        return False

    return filecmp.cmp(path1, path2)

def compare_analyze_results(dbname1: str, dbname2: str, query: int) -> bool:
    path1 = Path(f'data', 'mysql', dbname1, f'{query}.analyze.sql')
    path2 = Path(f'data', 'mysql', dbname2, f'{query}.analyze.sql')

    if not os.path.exists(path1):
        print(f'{COLORS.WARNING}Warning: Query {query} - Analyze file does not exists: {path1}{COLORS.ENDC}')
        return False
    if not os.path.exists(path2):
        print(f'{COLORS.WARNING}Warning: Query {query} - Analyze file does not exists: {path2}{COLORS.ENDC}')
        return False
    return cmp_analyze_results(path1,path2)

def cmp_analyze_results(path1: Path, path2: Path):
    # Calculate the difference
    regex = re.compile('(?P<indent>[ ]*)[->]+ (?P<name>[a-zA-Z-]+:?(?: [a-zA-Z-<>_]+){0,}){1}((\[[a-zA-Z0-9]+\]\((codegen=(?P<codegen>\d+\.\d+) compile=(?P<compile>\d+\.\d+)))|).*(time_spent_on_val_int_calls=(?P<val_int_calls>\d+\.\d+)).*\(actual time=(?P<actual_time_from>\d+\.\d+)..(?P<actual_time_to>\d+\.\d+) rows=(?P<rows>\d+) loops=(?P<loops>\d+)')
    
    access_paths = []
    for path in [path1, path2]:
        lines = read_file(path)
        names = []
        for line in lines:
            match = regex.match(line)
            if match is None:
                continue
            groups = match.groupdict()
            name = groups['name']
            if 'Filter:' in name:
                name = 'Filter:'
            names.append(name)
        access_paths.append(names)
    
    is_equal = '\n'.join(access_paths[0]) == '\n'.join(access_paths[1])

    # Print difference if the string was not equal
    if not is_equal:
        diff = list(set(access_paths[0]) - set(access_paths[1]))
        for item in diff:
            print(f'{COLORS.FAIL}+ {item}{COLORS.ENDC}')
        if len(diff) == 0:
            for i, x in enumerate(access_paths[0]):
                if i < len(access_paths[1]):
                    y = access_paths[1][i]
                    if x != y:
                        print(f'{COLORS.FAIL}+ {y}{COLORS.ENDC}')
                else:
                    print(f'{COLORS.FAIL}+ {x}{COLORS.ENDC}')
    return is_equal


if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument('dbname1', metavar='dbname1', type=str, help='dbname')
    parser.add_argument('dbname2', metavar='dbname2', type=str, help='dbname')
    parser.add_argument('--exclude', metavar='exclude', type=int, help='queries to exclude', nargs='+')
    parser.add_argument('--analyze', metavar='analyze', type=bool, help='should compare analyze')
    parser.add_argument('--test', metavar='test', type=bool, help='should compare analyze')
    args = parser.parse_args()

    print(f"Will compare: {args.dbname1} =? {args.dbname2}")
    print(f"Exclude: {args.exclude}")
    print(f"Mode: {'Results' if not args.analyze else 'Analyze'}")

    if args.test:
        from os import listdir
        from os.path import isfile, join
        onlyfiles = [f for f in listdir('data/mysql/m_tpch01_test_njit') if isfile(join('data/mysql/m_tpch01_test_njit', f))]
        onlyfiles = sorted(onlyfiles)
        first = None
        i = -1
        for f in onlyfiles:
            i += 1
            if '.test.sql' not in f:
                continue
            if i == 0:
                first = Path('data/mysql/m_tpch01_test_njit/', f)
                continue
            tmp = Path('data/mysql/m_tpch01_test_njit/', f)

            q = f.strip('.test.sql')
            is_equal = cmp_analyze_results(first, tmp)
            print(f"Query {q} is equal: {get_color(is_equal)}{is_equal}{COLORS.ENDC}")

        sys.exit(0)


    for i in range(1, 23):
        if args.exclude and i in args.exclude:
            continue
            
        if args.analyze:
            is_equal = compare_analyze_results(args.dbname1, args.dbname2, i)
        else:
            is_equal = compare_query_results(args.dbname1, args.dbname2, i)
        print(f"Query {i} is equal: {get_color(is_equal)}{is_equal}{COLORS.ENDC}")

        
